clear all

*---------------------------------------------------------------------------------------
* VALUE ADDED TAXES (VAT)
*---------------------------------------------------------------------------------------

// OECD data
import excel "data/raw/OECD_VAT Rates_extended.xlsx", sheet("2011") firstrow
foreach n of numlist 1968/1972 {
	gen vat`n' = .
}
forval n = 1968/2011 {
    replace vat`n' = 0 if `n'<implemented
}
forval n = 0/9 {
    replace country = subinstr(country, "`n'", "",.)
}

isocodes country, gen(iso3n) keepr(oecd)
drop  implemented Reduced-AU
tempfile vatmerge vatmerge2
save `vatmerge', replace

// handpicked data from the web
import excel "data/raw/additional_vat_data.xlsx", sheet("Sheet1") firstrow clear
isocodes country, gen(iso3n) keepr(oecd)
drop source country iso
save `vatmerge2', replace

// further EU data
import excel "data/raw/taxation_trends_report_2021_statutory_rates.xlsx", sheet("Table 2 -VAT rates") firstrow clear
drop if VATrate=="Reduced" |  VATrate=="Reduced " |  VATrate=="Reduced  "
replace vat2021 = "" if vat2021==":" 
destring vat*, replace
isocodes country, gen(iso3n) keepr(oecd)
drop Y VATrate 

// merge, clean, save
merge 1:1 iso3n using `vatmerge', nogen
merge 1:1 iso3n using `vatmerge2', nogen update
order iso3n vat*, alpha
drop country
save "data/processed/vatrates.dta", replace


*---------------------------------------------------------------------------------------
* IMF TAX POLICY REFORM DATA
*---------------------------------------------------------------------------------------

// import & prepare
import excel "data/raw/Tax Policy Reform Database v2.0 .xlsx", sheet("TPRD") firstrow clear

foreach x in month_announcement day_announcement year_announcement month_implementation day_implementation year_implementation TAX_major {
	cap replace `x' = "." if `x'=="x"
	destring `x', replace
}

// country variable
isocodes country, gen(iso3n) keepr(oecd)

// restrict data
keep if !mi(TAX_type) & !mi(TAX_reformtype) & TAX_major==1

// prepare tax change variables
gen pit_inc = 1 if (TAX_type=="PIT" | TAX_type=="SSC") & TAX_change=="INC"
gen pit_dec = 1 if (TAX_type=="PIT" | TAX_type=="SSC") & TAX_change=="DEC"
gen vat_inc = 1 if (TAX_type=="VAT" | TAX_type=="EXE") & TAX_change=="INC"
gen vat_dec = 1 if (TAX_type=="VAT" | TAX_type=="EXE") & TAX_change=="DEC"

// prepare time miss variables
gen pit_inc_miss = 0 if (TAX_type=="PIT" | TAX_type=="SSC") & TAX_change=="INC"
gen pit_dec_miss = 0 if (TAX_type=="PIT" | TAX_type=="SSC") & TAX_change=="DEC"
gen vat_inc_miss = 0 if (TAX_type=="VAT" | TAX_type=="EXE") & TAX_change=="INC"
gen vat_dec_miss = 0 if (TAX_type=="VAT" | TAX_type=="EXE") & TAX_change=="DEC"

foreach x in pit_inc_miss pit_dec_miss vat_inc_miss vat_dec_miss {
	replace `x' = 1 if !mi(year_implementation) & mi(month_implementation) & `x'==0
}

// time variable
replace month_implementation = 7 if !mi(year_implementation) & mi(month_implementation)
replace month_implementation = 7 if !mi(year_implementation) & mi(month_implementation)
gen ym_implement = (year_implementation*12) + month_implementation

// reshape and save
local indicators pit_inc pit_dec vat_inc vat_dec pit_inc_miss pit_dec_miss vat_inc_miss vat_dec_miss 
local indlist
local indicators2
foreach x in `indicators' {
	rename `x' RI`x'
	local indlist `indlist' (sum) RI`x'
	local indicators2 `indicators2' RI`x'
}
rename ym_implement ym
gcollapse `indlist', by(iso3n ym)
egen rnm = rownonmiss(`indicators2')
drop if mi(iso3n) | mi(ym) | mi(rnm) | rnm==0
keep iso3n ym `indicators2'
greshape wide `indicators2', by(iso3n) keys(ym)
save "data/processed/taxreforms_implement.dta", replace


*---------------------------------------------------------------------------------------
* CPDS DATA (for controls)
*---------------------------------------------------------------------------------------

*use "data/raw/CPDS_1960-2017_Update_2019.dta", clear
use "data/raw/CPDS_1960-2021_Update_2023.dta", clear
local CPDS_list unemp realgdpgr socspend deficit debt
drop iso3n
isocodes country, gen(iso3n) keepr(oecd)
rename socexp_t_pmp socspend
drop debt 
rename debt_hist debt 
keep iso3n year `CPDS_list'
greshape wide `CPDS_list', by(iso3n) keys(year)
save "data/processed/cpds_processed.dta",replace


*---------------------------------------------------------------------------------------
* EFFECTIVE WAGE TAXATION DATA
*---------------------------------------------------------------------------------------

// import & prep
use "data/raw/CWEP_Generosity_Scores_DATASET.dta", clear
isocodes COUNTRY, gen(iso3n) keepr(oecd)
rename YEAR year
xtset iso3n year
drop if year>2018

// ipolate 
sort iso3n year
foreach x in TAX_S50 TAX_S100 TAX_S150 TAX_S200 TAX_SP50 TAX_SP100 TAX_C1000 TAX_C5050 TAX_C10050 TAX_C150150 {
	by iso3n: ipolate `x' year, gen(`x'_ipo)
	replace `x' = `x'_ipo if mi(`x')
	replace `x' = `x'*100
}

// generate average tax rate variables
egen wagetax_av_help = rowmiss(TAX_S50 TAX_S100 TAX_S150 TAX_S200 TAX_SP50 TAX_SP100 TAX_C1000 TAX_C5050 TAX_C10050 TAX_C150150)
egen wagetax_av = rowmean(TAX_S50 TAX_S100 TAX_S150 TAX_S200 TAX_SP50 TAX_SP100 TAX_C1000 TAX_C5050 TAX_C10050 TAX_C150150) if wagetax_av_help==0
egen wagetax_av_alt_help = rowmiss(TAX_S100 TAX_SP100 TAX_C1000 TAX_C10050)
egen wagetax_av_alt = rowmean(TAX_S100 TAX_SP100 TAX_C1000 TAX_C10050) if wagetax_av_alt_help==0
gen wagetax_50 = TAX_S50
gen wagetax_100 = TAX_S100
gen wagetax_150 = TAX_S150
gen wagetax_200 = TAX_S200
gen wagetax_pc = TAX_C5050
gen wagetax_ac = TAX_C10050
gen wagetax_rc = TAX_C150150

// clean, reshape, and save
local CWED_list wagetax_av wagetax_av_alt wagetax_50 wagetax_100 wagetax_150 wagetax_200 wagetax_pc wagetax_ac wagetax_rc
keep iso3n year `CWED_list'
reshape wide `CWED_list',i(iso3n) j(year)
save "data/processed/cwed_processed.dta", replace


*---------------------------------------------------------------------------------------
* TAX REVENUE DATA
*---------------------------------------------------------------------------------------

import delimited "data\raw\oecd_tax_revenue.csv", clear 
isocodes country, gen(iso3n) keepr(oecd)
keep iso3n year value 
rename value taxrevenue
greshape wide taxrevenue, by(iso3n) keys(year)
save "data/processed/taxrevenue.dta",replace


*---------------------------------------------------------------------------------------
* CORPORATE TAXATION DATA
*---------------------------------------------------------------------------------------

import excel "data\raw\taxfoundation_corporate_inc_taxes.xlsx", sheet("1980-2022-Corporate-Tax-Rates-A") firstrow clear
isocodes country, gen(iso3n) keepr(oecd)
keep iso3n corptax*
foreach x of varlist corptax* {
	replace `x' = "" if `x'=="NA"
	destring `x', replace
}
save "data/processed/corptax.dta",replace
















