clear all

*---------------------------------------------------------------------------------------
* SET UP BASELINE DATASET FROM PARLGOV
*---------------------------------------------------------------------------------------

*--------------------------------------------
/* Holger Döring and Philip Manow, the creators of the ParlGov data, voiced the 
preference that we do not upload the raw ParlGov data as part of this replication
package. Therefore, the following code that creates the baseline dataset from the 
raw data is commented out, and the replication package uses a saved version of 
the baseline data created with this code ("parlgov_processed.dta"). To perfectly 
replicate the project, save the three sub-datasets of ParlGov (election, cabinet, 
party) in Stata format into the "data/raw" folder and run the following code. 
*--------------------------------------------

// prepare data from "future" elections 
local list vote_share edate
use "data/raw/parlgov_election.dta", clear
drop if election_type=="ep"
gen edate = date(election_date, "YMD")
format %td edate
sort party_id edate
rename election_id next_election_id 
rename previous_parliament_election_id election_id
foreach var in `list' {
	rename `var' next_`var'
}
keep party_id election_id next_*
drop if mi(election_id) | mi(next_election_id)
tempfile nextmerge 
save `nextmerge', replace

use "data/raw/parlgov_election.dta", clear
foreach num of numlist 1/3 {
	merge m:1 party_id election_id using `nextmerge', nogen keep(1 3)
	local numminus = `num'-1
	rename election_id election_id_t`numminus' 
	rename next_election_id election_id 
	foreach var in `list' {
		rename next_`var' `var'_t`num'
	}
}
rename election_id election_id_t3
rename election_id_t0 election_id
local keeplist
foreach var in `list' {
	local keeplist "`keeplist' `var'*"
}
keep party_id election_id `keeplist'
save "data/processed/parlgov_election_next.dta", replace

// load and prepare cabinet data
use "data/raw/parlgov_cabinet.dta", clear
isocodes country_name, gen(iso3n) keepr(oecd)
gen startdate = date(start_date, "YMD")
format %td startdate
gen edate = date(election_date, "YMD")
format %td edate
sort country_id edate startdate party_id
by country_id: gen enddate1 = (startdate[_n+1]-1) if startdate[_n+1]!=startdate
egen enddate = max(enddate1), by(country_id cabinet_id)
format %td enddate

// crucial variable generation before collapsing the data
*help variables
gen seatshare = cond(!mi(seats) & !mi(election_seats_total),seats/election_seats_total,0)
gegen govseats = sum(seatshare) if cabinet_party==1, by(iso3n cabinet_id)
gen seatshare_ingov = seatshare/govseats
merge m:1 party_id using "data/raw/parlgov_party.dta", nogen keepus(family_id) keep(1 3)
gen left = cond(inlist(family_id,19,11,14),1,0)
*minority gov
gen minority = cond(!mi(govseats) & govseats<.5,1,0) if cabinet_party==1
*effective no of parties
gegen maxseatshare = max(seatshare) if seatshare>0 & !mi(seatshare), by(iso3n cabinet_id)
gen effnum = 1/(1+((maxseatshare*maxseatshare)/seatshare)-seatshare)
gegen effnum_party = sum(effnum) if seatshare>0 & !mi(seatshare), by(iso3n cabinet_id)
*government cohesion
gegen helpcohesion = max(seatshare_ingov) if cabinet_party==1 & !mi(seatshare_ingov), by(iso3n cabinet_id)
gen dominantparty = 1 if seatshare_ingov==helpcohesion & cabinet_party==1
gen dominantpartyfam = left if dominantparty==1 & cabinet_party==1
gegen dominantpartycode = max(dominantpartyfam) if cabinet_party==1, by(iso3n cabinet_id)
replace dominantpartycode = . if dominantpartycode!=left
gegen ideo_cohesion_help = sum(seatshare_ingov) if !mi(dominantpartycode) & cabinet_party==1, by(iso3n cabinet_id dominantpartycode)
gegen ideo_cohesion = max(ideo_cohesion_help) if cabinet_party==1, by(iso3n cabinet_id)
*dominant party seat share
gen headshare_help = seatshare_ingov if dominantparty==1 & cabinet_party==1
gegen headshare = max(headshare_help) if cabinet_party==1, by(iso3n cabinet_id)
*cohabitation
gen cohabitation = 0 if cabinet_party==1
foreach x in "Klaus III" "Vranitzky I" "Silva I" "Silva II" "Silva III" "Passos Coelho I" "Passos Coelho II" ///
"Chirac II" "Balladur" "Jospin" "Tusk I" "Tusk II" "Schuessel IV" "Aho I" "Aho II" "Vanhanen II" ///
"Cosgrave L" "Fitzgerald I" "Fitzgerald II" "Haughey IV" "Reynolds I" "Kenny II" "Barroso" {
	replace cohabitation = 1 if cabinet_name=="`x'" & cabinet_party==1
}

// create baseline dataset of party-governments
keep if cabinet_party==1 & caretaker!=1 & !mi(startdate) & !mi(enddate)
collapse (min) startdate edate (max) enddate minority effnum_party cohabitation ideo_cohesion headshare prime_minister (first) country_name seats election_seats_total party_name_short party_name party_name_english , by(iso3n election_id party_id)
sort iso3n startdate party_id 

// merge other ParlGov datasets
merge m:1 party_id using "data/raw/parlgov_party.dta", nogen keepus(family_name family_id state_market cmp) keep(1 3)
merge 1:1 party_id election_id using "data/processed/parlgov_election_next.dta", nogen keep(1 3)

save "data/raw/parlgov_processed", replace

*/

use "data/raw/parlgov_processed.dta", clear

*---------------------------------------------------------------------------------------
* MERGE FURTHER DATA 
*---------------------------------------------------------------------------------------

merge m:1 iso3n using "data/processed/cpds_processed.dta", keep(1 3) nogen
merge m:1 iso3n using "data/processed/vatrates.dta", keep(1 3) nogen
merge m:1 iso3n using "data/processed/taxreforms_implement.dta", keep(1 3) nogen
merge m:1 iso3n using "data/processed/cwed_processed.dta", keep(1 3) nogen
merge m:1 iso3n using "data/processed/taxrevenue.dta", keep(1 3) nogen
merge m:1 iso3n using "data/processed/corptax.dta", keep(1 3) nogen
gsort iso3n startdate party_name_english 


*---------------------------------------------------------------------------------------
* PARTY AND GOVERNMENT VARIABLES
*---------------------------------------------------------------------------------------

// start and end dates of governments
replace enddate = edate_t1 if edate_t1<enddate
gen startyear = year(startdate)
gen endyear = year(enddate)
gen totyears = endyear-startyear 
gen days = enddate-startdate
gen logdays = log(days)
foreach d in start end {
    foreach t in year month day {
		gen `d'_`t' = `t'(`d'date)
	}
	gen `d'date_ym = (`d'_year*12)+(`d'_month)
	drop `d'_year `d'_month `d'_day
}

// vote change
gen votechange = vote_share_t1-vote_share
gen votechange_pc = ((vote_share_t1-vote_share)/vote_share)*100

// number of cabinet parties
bys iso3n edate startdate: egen no_cabparties = count(days)
*bys iso3n edate startdate party_id: egen no_cabparties = mean(no_cabparties1)  // this should be weighted by days, but not needed for our purpose
gen singlepartygov = cond(no_cabparties==1 & minority!=1,1,0) if !mi(no_cabparties)
gen dualpartygov = cond(inrange(no_cabparties,1,2),1,0) if !mi(no_cabparties)
drop no_cabparties

// party family variables
gen parfam = .
replace parfam = 1 if family_id==19 
replace parfam = 2 if family_id==11 | family_id==14
replace parfam = 3 if family_id==6
replace parfam = 4 if family_id==3
replace parfam = 5 if family_id==26 | family_id==40
replace parfam = 6 if family_id==2
replace parfam = 7 if family_id==12
replace parfam = 99 if family_id==16
lab def partyfam 1 "Ecologist" 2 "Social dem. & socialist" 3 "Liberal" ///
	4 "Christian democrat" 5 "Conservative & right-wing" 6 "Agrarian" ///
	7 "Other" 99 "Special issue"
lab val parfam partyfam
recode parfam (1 2=1) (3 4 5 6 7 99=0), gen(left)
recode parfam (1 2 4=1) (3 5 6 7 99=0), gen(left_alt)
recode parfam (3 5=1) (1 2 4 6 7 99=0), gen(right)
recode parfam (3 4 5=1) (1 2 6 7 99=0), gen(right_alt)
drop family_name family_id

// time identifier variable
foreach i of numlist 1969 1975(5)2020 {
	local yearlist `yearlist'`i' 
	if `i'!=2020 local yearlist `yearlist',
}
gen time_fe = irecode(endyear,`yearlist')
tab time_fe, gen(time_fe)
gen time_fealt = irecode(startyear,`yearlist')
tab time_fealt, gen(time_fealt)


*---------------------------------------------------------------------------------------
* POLICY CHANGE AND REFORM EVENT VARIABLES
*---------------------------------------------------------------------------------------

// changes from first to last gov year
sort iso3n startdate party_id
qui sum totyears
local maxtotyear = r(max)
foreach y in unemp realgdpgr vat wagetax_av wagetax_av_alt wagetax_50 wagetax_100 wagetax_150 wagetax_200 wagetax_pc wagetax_ac wagetax_rc bot_marg top_marg deficit debt taxrevenue corptax {
    
	local `y'list
	foreach govy of numlist 1/`maxtotyear' {
		gen `y'`govy' = .
		local `y'list ``y'list' `y'`govy'
	}
	foreach i of numlist 1960/2020 {
		foreach govy of numlist 1/`maxtotyear' {
			local n = `govy'-1
			cap replace `y'`govy' = `y'`i' if `i'==(startyear+`n') & endyear>=(startyear+`n') & !mi(endyear)
		}
	}
	gen D`y' = .
	gen D`y'_sh = .
	foreach govy of numlist 2/`maxtotyear' {
		local n = `govy'-1
		replace D`y' = `y'`govy'-`y'1 if totyears==`n'
		replace D`y'_sh = `y'`govy'-`y'`n' if totyears==`n'
	}
	egen `y'_avg = rowmean(``y'list')
	clonevar `y'_y1 = `y'1
	drop ``y'list'
}

// reform implementations
foreach x in RIpit_inc RIpit_dec RIvat_inc RIvat_dec {
	gen `x' = 0 if startyear>=1970 & endyear<=2014 & ///
	inlist(iso3n,36,40,76,124,156,203,208,250,276,300,356,372,380,392,410,442,484,616,620,724,792,826,840)
	foreach year of numlist 1946/2017 {
		foreach month of numlist 1/12 {
			local comb = (`year'*12)+`month'
			cap replace `x' = `x'+`x'`comb' if !mi(`x'`comb') & `comb'>=startdate_ym & `comb'<=enddate_ym
			cap replace `x' = `x'-`x'_miss`comb' if !mi(`x'_miss`comb') & `comb'>=startdate_ym & `comb'<=enddate_ym & (startyear==`year' | endyear==`year')
		}
	}
}

foreach x in pit vat {
	gen RI`x' = RI`x'_inc - RI`x'_dec
}

foreach x in RIpit_inc RIpit_dec RIvat_inc RIvat_dec {
	gen `x'_sh = 0 if startyear>=1970 & endyear<=2014 & ///
	inlist(iso3n,36,40,76,124,156,203,208,250,276,300,356,372,380,392,410,442,484,616,620,724,792,826,840)
	foreach year of numlist 1946/2017 {
		foreach month of numlist 1/12 {
			local comb = (`year'*12)+`month'
			local comb_year = ((`year'*12)+`month')+12
			cap replace `x'_sh = `x'+`x'`comb' if !mi(`x'`comb') & `comb'>=startdate_ym & `comb'<=enddate_ym  & `comb_year'>=enddate_ym
			cap replace `x'_sh = `x'_-`x'_miss`comb' if !mi(`x'_miss`comb') & `comb'>=startdate_ym & `comb'<=enddate_ym  & `comb_year'>=enddate_ym  & (startyear==`year' | endyear==`year')
		}
	}
}

foreach x in pit vat {
	rename RI`x'_inc_sh RI`x'_sh_inc
	rename RI`x'_dec_sh RI`x'_sh_dec
	gen RI`x'_sh = RI`x'_sh_inc - RI`x'_sh_dec
}

// drop all superfluous variables
drop unemp1960-corptax2022


*---------------------------------------------------------------------------------------
* SAMPLE RESTRICTION
*---------------------------------------------------------------------------------------

gsort iso3n startdate party_id
keep if startyear>=1970 & days>=365 & !mi(days) & vote_share>0 & !mi(vote_share) & !mi(votechange) & ///
(!mi(Dvat) | !mi(Dwagetax_av) | !mi(Dtop_marg) | !mi(Dbot_marg) | !mi(RIpit) | !mi(RIvat))

gen keep3 = 1
foreach x in Dunemp unemp_avg Drealgdpgr realgdpgr_avg deficit_y1 debt_y1 {
    replace keep3 = 0 if mi(`x')
}
drop if keep3!=1
drop keep3


*---------------------------------------------------------------------------------------
* EDIT VARIABLE VALUES
*---------------------------------------------------------------------------------------

// make short-term variables have same sample 
foreach x in Dvat {
	replace `x'_sh = . if mi(`x')
}
foreach x of varlist Dwagetax* {
	replace `x' = . if mi(Dwagetax_av)
}

// drop values with no variation within countries
foreach x in votechange votechange_pc RIpit RIvat Dwagetax_av Dwagetax_av_alt Dvat Dwagetax_50 Dwagetax_100 Dwagetax_150 Dwagetax_200 Dwagetax_pc Dwagetax_ac Dwagetax_rc Dtaxrevenue Dcorptax {
	gegen `x'_var = sd(`x'), by(iso3n)
	replace `x' = . if `x'_var==0
	drop `x'_var
}

// outlier values
foreach x in votechange votechange_pc Dwagetax_av Dwagetax_av_alt Dvat Dwagetax_50 Dwagetax_100 Dwagetax_150 Dwagetax_200 Dwagetax_pc Dwagetax_ac Dwagetax_rc Dtaxrevenue Dcorptax Dtaxrevenue_sh Dcorptax_sh days vote_share effnum_party unemp_avg Dunemp realgdpgr_avg Drealgdpgr deficit_y1 debt_y1 {
	clonevar `x'_og = `x'
}
clonevar Dwagetax_av_og_sh = Dwagetax_av_sh
clonevar Dvat_og_sh = Dvat_sh
foreach ref in RIvat RIpit {
	foreach o in " " _inc _dec {
		clonevar `ref'_og`o' = `ref'`o'
		clonevar `ref'_og_sh`o' = `ref'_sh`o'
	}
}

gstats winsor votechange votechange_pc RIpit RIvat RIpit_inc RIvat_inc RIpit_dec RIvat_dec RIpit_sh RIvat_sh RIpit_sh_inc RIvat_sh_inc RIpit_sh_dec RIvat_sh_dec Dwagetax_av Dwagetax_av_sh Dwagetax_av_alt Dvat_sh Dwagetax_50 Dwagetax_100 Dwagetax_150 Dwagetax_200 Dwagetax_pc Dwagetax_ac Dwagetax_rc Dtaxrevenue Dcorptax Dtaxrevenue_sh Dcorptax_sh days vote_share effnum_party unemp_avg Dunemp realgdpgr_avg Drealgdpgr deficit_y1 debt_y1, replace cuts(.5 99.5)
gstats winsor Dvat, replace cuts(0 97)

// make coefficients legible
replace days = days/1000


*---------------------------------------------------------------------------------------
* FINALIZE VARIABLES (after arriving at final sample)
*---------------------------------------------------------------------------------------

// clarity of responsibility variables
foreach x in ideo_cohesion headshare {
	qui sum `x'
	replace `x' = (`x'-r(min))*1/(r(max)-r(min))
}
gen clarity = (ideo_cohesion+headshare+singlepartygov+cohabitation)/4
qui sum clarity,d 
gen clarity_dum = clarity>=r(p50) & !mi(clarity)
drop ideo_cohesion headshare singlepartygov cohabitation dualpartygov

// weight
egen no_cabparties = count(days), by(iso3n edate startdate)
gen cabparties_w = 1/no_cabparties 
gegen mean_cabparties_w = mean(cabparties_w)
replace cabparties_w = cabparties_w/mean_cabparties_w
gegen mean_voteshare = mean(vote_share), by(iso3n edate startdate)
gen voteshare_w = vote_share/mean_voteshare
gen cabp_votesh_w = cabparties_w*voteshare_w
gegen mean_days = mean(days), by(iso3n edate)
gen days_w = days/mean_days
drop no_cabparties mean_cabparties_w mean_voteshare mean_days

// fiscal pressure identifier
gen low_pressure = cond(deficit_y1>=0 | debt_y1<=30,1,0) if !mi(deficit_y1)  & !mi(debt_y1)
gen high_pressure = cond(deficit_y1<-3,1,0) if !mi(deficit_y1)

// tax increase vs decrease
foreach x in Dvat Dvat_sh Dwagetax_av Dwagetax_av_sh Dwagetax_av_alt Dwagetax_av_alt_sh Dwagetax_50 Dwagetax_50_sh ///
Dwagetax_100 Dwagetax_100_sh Dwagetax_150 Dwagetax_150_sh Dwagetax_200 ///
Dwagetax_200_sh Dwagetax_pc Dwagetax_pc_sh Dwagetax_ac Dwagetax_ac_sh Dwagetax_rc ///
Dwagetax_rc_sh Dtop_marg Dtop_marg_sh Dbot_marg Dbot_marg_sh ///
Dcorptax Dcorptax_sh Dtaxrevenue Dtaxrevenue_sh ///
Dvat_og Dvat_og_sh Dwagetax_av_og Dwagetax_av_og_sh {
	gen `x'_dec = cond(`x'<=0,abs(`x'),0) if !mi(`x')
	gen `x'_inc = cond(`x'>=0,abs(`x'),0) if !mi(`x')
}


*---------------------------------------------------------------------------------------
* VARIABLE LABELS
*---------------------------------------------------------------------------------------

cap program drop label_leo
program label_leo
cap lab var votechange "Vote change (pp.)"
cap lab var votechange_pc "Vote change (%)"

cap lab var Dvat "VAT rate change" 
cap lab var Dvat_inc "VAT rate increase" 
cap lab var Dvat_dec "VAT rate decrease" 
cap lab var Dvat_sh "Short-term VAT rate increase" 
cap lab var Dvat_sh_inc "Short-term VAT rate inc" 
cap lab var Dvat_sh_dec "Short-term VAT rate decrease" 

cap lab var Dvat_og "VAT rate change" 
cap lab var Dvat_og_inc "VAT rate increase" 
cap lab var Dvat_og_dec "VAT rate decrease" 
cap lab var Dvat_og_sh "Short-term VAT rate increase" 
cap lab var Dvat_og_sh_inc "Short-term VAT rate inc" 
cap lab var Dvat_og_sh_dec "Short-term VAT rate decrease" 

cap lab var Dwagetax_av "Average wage tax change" 
cap lab var Dwagetax_av_inc "Average wage tax increase" 
cap lab var Dwagetax_av_dec "Average wage tax decrease" 
cap lab var Dwagetax_av_sh "Short-term average wage tax change" 
cap lab var Dwagetax_av_sh_inc "Short-term average wage tax increase" 
cap lab var Dwagetax_av_sh_dec "Short-term average wage tax decrease" 

cap lab var Dwagetax_av_og "Average wage tax change" 
cap lab var Dwagetax_av_og_inc "Average wage tax increase" 
cap lab var Dwagetax_av_og_dec "Average wage tax decrease" 
cap lab var Dwagetax_av_og_sh "Short-term average wage tax change" 
cap lab var Dwagetax_av_og_sh_inc "Short-term average wage tax increase" 
cap lab var Dwagetax_av_og_sh_dec "Short-term average wage tax decrease" 

cap lab var Dwagetax_50 "50% earnings single tax change" 
cap lab var Dwagetax_50_inc "50% earnings single tax increase" 
cap lab var Dwagetax_50_dec "50% earnings single tax decrease" 
cap lab var Dwagetax_50_sh "Short-term 50% earnings single tax change" 
cap lab var Dwagetax_50_sh_inc "Short-term 50% earnings single tax increase" 
cap lab var Dwagetax_50_sh_dec "Short-term 50% earnings single tax decrease"

cap lab var Dwagetax_100 "100% earnings single tax change" 
cap lab var Dwagetax_100_inc "100% earnings single tax increase" 
cap lab var Dwagetax_100_dec "100% earnings single tax decrease" 
cap lab var Dwagetax_100_sh "Short-term 100% earnings single tax change" 
cap lab var Dwagetax_100_sh_inc "Short-term 100% earnings single tax increase" 
cap lab var Dwagetax_100_sh_dec "Short-term 100% earnings single tax decrease"

cap lab var Dwagetax_150 "150% earnings single tax change" 
cap lab var Dwagetax_150_inc "150% earnings single tax increase" 
cap lab var Dwagetax_150_dec "150% earnings single tax decrease" 
cap lab var Dwagetax_150_sh "Short-term 150% earnings single tax change" 
cap lab var Dwagetax_150_sh_inc "Short-term 150% earnings single tax increase" 
cap lab var Dwagetax_150_sh_dec "Short-term 150% earnings single tax decrease"

cap lab var Dwagetax_200 "200% earnings single tax change" 
cap lab var Dwagetax_200_inc "200% earnings single tax increase" 
cap lab var Dwagetax_200_dec "200% earnings single tax decrease" 
cap lab var Dwagetax_200_sh "Short-term 200% earnings single tax change" 
cap lab var Dwagetax_200_sh_inc "Short-term 200% earnings single tax increase" 
cap lab var Dwagetax_200_sh_dec "Short-term 200% earnings single tax decrease"

cap lab var Dwagetax_pc "50%-50% earnings couple tax change" 
cap lab var Dwagetax_pc_inc "50%-50% earnings couple tax increase" 
cap lab var Dwagetax_pc_dec "50%-50% earnings couple tax decrease" 
cap lab var Dwagetax_pc_sh "Short-term 50%-50% earnings couple tax change" 
cap lab var Dwagetax_pc_sh_inc "Short-term 50%-50% earnings couple tax increase" 
cap lab var Dwagetax_pc_sh_dec "Short-term 50%-50% earnings couple tax decrease"

cap lab var Dwagetax_ac "100%-50% earnings couple tax change" 
cap lab var Dwagetax_ac_inc "100%-50% earnings couple tax increase" 
cap lab var Dwagetax_ac_dec "100%-50% earnings couple tax decrease" 
cap lab var Dwagetax_ac_sh "Short-term 100%-50% earnings couple tax change" 
cap lab var Dwagetax_ac_sh_inc "Short-term 100%-50% earnings couple tax increase" 
cap lab var Dwagetax_ac_sh_dec "Short-term 100%-50% earnings couple tax decrease"

cap lab var Dwagetax_rc "150%-150% earnings couple tax change" 
cap lab var Dwagetax_rc_inc "150%-150% earnings couple tax increase" 
cap lab var Dwagetax_rc_dec "150%-150% earnings couple tax decrease" 
cap lab var Dwagetax_rc_sh "Short-term 150%-150% earnings couple tax change" 
cap lab var Dwagetax_rc_sh_inc "Short-term 150%-150% earnings couple tax increase" 
cap lab var Dwagetax_rc_sh_dec "Short-term 150%-150% earnings couple tax decrease"

cap lab var RIpit "Net PIT reforms"
cap lab var RIpit_inc "PIT increase reforms"
cap lab var RIpit_dec "PIT decrease reforms"
cap lab var RIpit_sh "Net short-term PIT reforms"
cap lab var RIpit_sh_inc "Short-term PIT increase reforms"
cap lab var RIpit_sh_dec "Short-term PIT decrease reforms"

cap lab var RIpit_og "Net PIT reforms"
cap lab var RIpit_og_inc "PIT increase reforms"
cap lab var RIpit_og_dec "PIT decrease reforms"
cap lab var RIpit_og_sh "Net short-term PIT reforms"
cap lab var RIpit_og_sh_inc "Short-term PIT increase reforms"
cap lab var RIpit_og_sh_dec "Short-term PIT decrease reforms"

cap lab var RIvat "Net VAT reforms"
cap lab var RIvat_inc "VAT increase reforms"
cap lab var RIvat_dec "VAT decrease reforms"
cap lab var RIvat_sh "Net short-term VAT reforms"
cap lab var RIvat_sh_inc "Short-term VAT increase reforms"
cap lab var RIvat_sh_dec "Short-term VAT decrease reforms"

cap lab var RIvat_og "Net VAT reforms"
cap lab var RIvat_og_inc "VAT increase reforms"
cap lab var RIvat_og_dec "VAT decrease reforms"
cap lab var RIvat_og_sh "Net short-term VAT reforms"
cap lab var RIvat_og_sh_inc "Short-term VAT increase reforms"
cap lab var RIvat_og_sh_dec "Short-term VAT decrease reforms"

cap lab var Dcorptax "Corporate tax change" 
cap lab var Dcorptax_inc "Corporate tax increase" 
cap lab var Dcorptax_dec "Corporate tax decrease" 
cap lab var Dcorptax_sh "Short-term corporate tax change" 
cap lab var Dcorptax_sh_inc "Short-term corporate tax increase" 
cap lab var Dcorptax_sh_dec "Short-term corporate tax decrease" 

cap lab var Dtaxrevenue "Tax revenue change" 
cap lab var Dtaxrevenue_inc "Tax revenue increase" 
cap lab var Dtaxrevenue_dec "Tax revenue decrease" 
cap lab var Dtaxrevenue_sh "Short-term tax revenue change" 
cap lab var Dtaxrevenue_sh_inc "Short-term tax revenue increase" 
cap lab var Dtaxrevenue_sh_dec "Short-term tax revenue decrease" 

cap lab var Dunemp "Unemployment change"
cap lab var unemp_avg "Average unemployment"
cap lab var Drealgdpgr "GDP growth change"
cap lab var realgdpgr_avg "Average GDP growth"
cap lab var deficit_y1 "Initial household balance"
cap lab var debt_y1 "Initial debt level"
cap lab var effnum_party "Effective number of parties"
cap lab var logdays "Government duration (log days)"
cap lab var days "Government duration in days"
cap lab var vote_share "Initial vote share"
cap lab var left "Left government party"
cap lab var left_alt "Left government party (including CDs)"
cap lab var right "Right government party"
cap lab var clarity_dum "High clarity of responsibility"
cap lab var low_pressure "Low fiscal pressure"
cap lab var high_pressure "High fiscal pressure"
end
label_leo

*---------------------------------------------------------------------------------------
* COUNT HOW MUCH IS INCLUDED IN THE FINAL DATASET
*---------------------------------------------------------------------------------------

egen gov_id = group(iso3n edate)
egen partycount = tag(party_id)
egen electioncount = tag(iso3n edate)
egen countrycount = tag(iso3n)
qui count
dis r(N) " observations overall"
qui count if partycount==1
dis "from " r(N) " parties"
qui count if electioncount==1
dis "from " r(N) " governments"
qui count if countrycount==1
dis "from " r(N) " countries"
drop partycount electioncount countrycount


*---------------------------------------------------------------------------------------
* SAVE CLEAN DATA
*---------------------------------------------------------------------------------------

// party-government main dataset
sort iso3n party_id startdate 
save "data/processed/data_votechange.dta", replace

// government dataset
preserve 
foreach ind in RIpit RIvat Dwagetax_av Dvat Dwagetax_50 Dwagetax_100 Dwagetax_200 Dwagetax_pc Dwagetax_ac Dwagetax_rc Dtaxrevenue Dcorptax {
	foreach variant in "" _inc _dec {
		foreach variant2 in "" _sh {
			local vlist `vlist' `ind'`variant2'`variant'
		}
	}
}
gcollapse (sum) votechange votechange_pc (mean) `vlist' days vote_share effnum_party unemp_avg Dunemp realgdpgr_avg Drealgdpgr deficit_y1 debt_y1 left right clarity_dum high_pressure clarity [w=days_w], by(iso3n edate)
label_leo
gen year = year(edate)
foreach i of numlist 1968 1975(5)2010 {
	local yearlist2 `yearlist2'`i' 
	if `i'!=2010 local yearlist2 `yearlist2',
}
gen time_fe = irecode(year,`yearlist2')
save "data/processed/data_votechange_collapsed.dta", replace
restore











